use anyhow::Result;
use async_graphql::SimpleObject;
use serde::{Deserialize, Serialize};
use sqlx::{FromRow, PgPool};

#[derive(SimpleObject, FromRow, Deserialize, Serialize)]
pub struct Modell {
    /// Die Datenbank-ID
    pub modell_id: i32,

    /// Der Typ
    typ_id: i32,

    /// Der Hersteller
    hersteller_id: i32,

    /// Der Name eines Modells
    name: String,
}

impl Modell {
    pub async fn create(
        pool: &PgPool,
        typ_id: i32,
        hersteller_id: i32,
        name: &str,
    ) -> Result<Modell> {
        let row = sqlx::query!(
            "INSERT INTO modelle(typ_id, hersteller_id, name) VALUES ($1, $2, $3) RETURNING modell_id",
            typ_id,
            hersteller_id,
            name
        )
        .fetch_one(pool)
        .await?;

        Ok(Modell {
            modell_id: row.modell_id,
            typ_id,
            hersteller_id,
            name: name.to_string(),
        })
    }

    pub async fn read_one(pool: &PgPool, id: &i32) -> Result<Modell> {
        let row = sqlx::query_as!(Modell, "SELECT * FROM modelle WHERE modell_id = $1", id)
            .fetch_one(pool)
            .await?;

        Ok(row)
    }

    pub async fn read_all(pool: &PgPool) -> Result<Vec<Modell>> {
        let rows = sqlx::query_as!(
            Modell,
            "SELECT modell_id, name, hersteller_id, typ_id FROM modelle"
        )
        .fetch_all(pool)
        .await?;

        Ok(rows)
    }

    pub async fn update(pool: &PgPool, id: &i32, name: &str) -> Result<Modell> {
        sqlx::query!("UPDATE modelle SET name=$1 WHERE modell_id = $2", name, id)
            .execute(pool)
            .await?;

        let t = Modell::read_one(pool, id).await?;

        Ok(t)
    }

    pub async fn delete(pool: &PgPool, id: &i32) -> Result<()> {
        sqlx::query!("DELETE FROM modelle WHERE modell_id = $1", id)
            .execute(pool)
            .await?;

        Ok(())
    }
}
