use anyhow::Result;
use async_graphql::SimpleObject;
use serde::{Deserialize, Serialize};
use sqlx::{FromRow, PgPool};

use super::{hersteller::Hersteller, typ::Typ};

#[derive(SimpleObject, Debug, FromRow, Deserialize, Serialize)]
pub struct ModellTemp {
    /// Die Datenbank-ID
    pub modell_id: i32,

    /// Der Typ
    pub typ: Typ,

    /// Der Hersteller
    pub hersteller: Hersteller,

    /// Der Name eines Modells
    pub name: String,
}

impl ModellTemp {
    pub async fn read_one(pool: &PgPool, id: &i32) -> Result<ModellTemp> {
        let row = sqlx::query_as!(
            ModellTemp,
            r#"
             select
               m.modell_id, m.name,
               (t.typ_id, t.name) as "typ!: Typ",
               (h.hersteller_id, h.name) as "hersteller!: Hersteller"
             from modelle as m
             join typen as t using(typ_id)
             join hersteller as h using(hersteller_id)
             where m.modell_id = $1;
         "#,
            id
        )
        .fetch_one(pool)
        .await?;

        Ok(row)
    }
}
